package com.tenlionsoft.aimz_k.page.activity

import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.net.Uri
import android.os.Build
import android.provider.Settings
import androidx.activity.OnBackPressedCallback
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import com.tenlionsoft.aimz_k.R
import com.tenlionsoft.aimz_k.adapter.VpAdapter
import com.tenlionsoft.aimz_k.databinding.ActivityMainBinding
import com.tenlionsoft.aimz_k.page.fragments.CenterFragment
import com.tenlionsoft.aimz_k.page.fragments.MineFragment
import com.tenlionsoft.aimz_k.page.fragments.MsgFragment
import com.tenlionsoft.aimz_k.viewmodel.MainPageViewModel
import com.tenlionsoft.baselib.base.BaseActivity
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.net.DownloadApkService
import com.tenlionsoft.baselib.utils.AppUtils
import com.tenlionsoft.baselib.utils.DensityUtils
import com.tenlionsoft.baselib.utils.LogUtils
import com.tenlionsoft.baselib.utils.ToastUtils
import com.tenlionsoft.baselib.widget.CenterProgressUpdateView
import java.io.File

/**
 * 首页
 */
class MainActivity : BaseActivity() {
    private lateinit var mBinding: ActivityMainBinding
    private lateinit var mFragments: ArrayList<Fragment>
    private var exitTime: Long = 0
    private var mCurPosition: Int = 0
    private val mActions = arrayOf<String>(
        ProjectConfig.ACTION_UPDATE_SUCCESS,
        ProjectConfig.ACTION_UPDATE_ERROR,
        ProjectConfig.ACTION_UPDATE_START,
        ProjectConfig.ACTION_UPDATE_PROGRESS
    )
    private lateinit var mLocalReceiver: MainBroadcastReceiver
    private var mUpdateView: CenterProgressUpdateView? = null
    private var mApkFile: File? = null
    private var onSocketConnectListener: OnSocketConnectListener? = null
    private val mainPageViewModel: MainPageViewModel by lazy {
        ViewModelProvider(this)[MainPageViewModel::class.java]
    }

    override fun bindView() {
        mBinding = DataBindingUtil.setContentView(this, R.layout.activity_main);
        mBinding.mainModel = mainPageViewModel
        initView();
    }

    /**
     * 初始化页面
     */
    private fun initView() {
        mFragments = ArrayList();
        mFragments.add(MsgFragment.newInstance());
        mFragments.add(CenterFragment.newInstance());
        mFragments.add(MineFragment.newInstance());
        mBinding.vpContent.adapter = VpAdapter(this, mFragments);
        mBinding.bnvTab.itemTextColor = resources.getColorStateList(R.color.col_tabs, theme);
        mBinding.bnvTab.itemIconSize = DensityUtils.dp2px(this, 24F);
        mBinding.vpContent.isUserInputEnabled = false;
        mBinding.bnvTab.setOnItemSelectedListener { item ->
            when (item.itemId) {
                R.id.menu_item_msg -> {
                    setStatusBarColor(true);
                    mBinding.vpContent.setCurrentItem(0, false);
                    mCurPosition = 0;
                    return@setOnItemSelectedListener true;
                }//首页
                R.id.menu_item_center -> {
                    setStatusBarColor(true);
                    mBinding.vpContent.setCurrentItem(1, false);
                    mCurPosition = 1;
                    return@setOnItemSelectedListener true;
                }//统计
                R.id.menu_item_mine -> {
                    setStatusBarColor(false);
                    mBinding.vpContent.setCurrentItem(2, false);
                    mCurPosition = 2;
                    return@setOnItemSelectedListener true;
                }//我的
            }
            false;
        }

        onBackPressedDispatcher.addCallback(this, object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
                if (mCurPosition == 0) {
                    if ((System.currentTimeMillis() - exitTime) > 2000) {
                        ToastUtils.normal("再按一次退出程序")
                        exitTime = System.currentTimeMillis();
                    } else {
                        exitApp();
                    }
                } else {
                    mBinding.bnvTab.selectedItemId = R.id.menu_item_msg
                }
            }
        })
        mainPageViewModel.doCheckAppVersion(this@MainActivity)
        mainPageViewModel.isNeedDownload.observe(this) {
            if (it) {
                startDownloadApk()
            }
        }
    }


    private fun unRegisterLocalReceiver() {
        unregisterReceiver(mLocalReceiver)
    }

    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    private fun registerLocalReceiver() {
        mLocalReceiver = MainBroadcastReceiver()
        val filter = IntentFilter()
        filter.addAction(ProjectConfig.ACTION_UPDATE_SUCCESS) //下载成功
        filter.addAction(ProjectConfig.ACTION_UPDATE_ERROR) //下载失败
        filter.addAction(ProjectConfig.ACTION_UPDATE_PROGRESS) //进度更新
        filter.addAction(ProjectConfig.ACTION_UPDATE_START) //开始下载App
        filter.addAction(ProjectConfig.A_S_CONNECTED)//socket连接成功
        filter.addAction(ProjectConfig.A_S_FAIL)//socket连接失败
        filter.addAction(ProjectConfig.A_S_DISCONNECT)//socket连接断开
        filter.addAction(ProjectConfig.A_S_MSG_RECEIVER)//监听消息收到
        registerReceiver(mLocalReceiver, filter)
    }

    private fun startDownloadApk() {
        ToastUtils.normal("检测到新版本,开始下载")
        val intent = Intent(this@MainActivity, DownloadApkService::class.java)
        intent.putExtra(ProjectConfig.APK_DOWNLOAD_URL, ProjectConfig.APP_DOWNLOAD_URL)
        startService(intent)
    }


    private fun showUpdateProgress() {
        mUpdateView = CenterProgressUpdateView.DialogBuilder(this@MainActivity)
            .build()
        mUpdateView?.show()
    }

    /**
     * 安装Apk
     *
     * @param file
     */
    private fun installApk(file: File) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O && Build.VERSION.SDK_INT < Build.VERSION_CODES.R) {
            val b = packageManager.canRequestPackageInstalls()
            if (b) {
                AppUtils.installAPk(this@MainActivity, file)
            } else {
                //申请权限
                mApkFile = file
                val packageURI = Uri.parse("package:" + this@MainActivity.getPackageName())
                val intent = Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, packageURI)
                startActivityForResult(intent, 298)
            }
        } else {
            AppUtils.installAPk(this@MainActivity, file)
        }
    }

    override fun onStart() {
        super.onStart()
        registerLocalReceiver()
    }

    override fun onPause() {
        super.onPause()
        unRegisterLocalReceiver()
    }


    fun setOnSocketListener(onSocketConnectListener: OnSocketConnectListener) {
        this.onSocketConnectListener = onSocketConnectListener
    }

    inner class MainBroadcastReceiver : BroadcastReceiver() {

        override fun onReceive(context: Context?, intent: Intent?) {
            when (intent?.action) {
                //开始下载
                ProjectConfig.ACTION_UPDATE_START -> {
                    showUpdateProgress()
                }
                //更新进度
                ProjectConfig.ACTION_UPDATE_PROGRESS -> {
                    val progress = intent.getIntExtra("progress", 0)
                    LogUtils.e("当前进度:$progress")
                    mUpdateView?.setCurrentProgress(progress)
                }
                //下载失败
                ProjectConfig.ACTION_UPDATE_ERROR -> {
                    //下载失败
                    mUpdateView?.dismiss()
                }
                //下载成功
                ProjectConfig.ACTION_UPDATE_SUCCESS -> {
                    //下载成功
                    mUpdateView?.dismiss()
                    //判断是否可以安装
                    val apkFile = intent.getSerializableExtra("apkFile") as File?
                    installApk(apkFile!!)
                }
                //socket 连接成功
                ProjectConfig.A_S_CONNECTED -> {
                    onSocketConnectListener?.onListener(ProjectConfig.A_S_CONNECTED)
                }
                //连接失败
                ProjectConfig.A_S_FAIL -> {
                    onSocketConnectListener?.onListener(ProjectConfig.A_S_FAIL)
                }
                //接收到信息
                ProjectConfig.A_S_MSG_RECEIVER -> {
                    val extra = intent.getStringExtra("msg")
                    if (!extra.isNullOrEmpty()) {
                        onSocketConnectListener?.onReceiverMsg(extra)
                    }
                }

                else -> {}
            }
        }
    }

    interface OnSocketConnectListener {
        fun onListener(status: String)
        fun onReceiverMsg(msg: String)
    }
}


