package com.tenlionsoft.aimz_k.socket

import android.content.Context
import android.content.Intent
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.AesUtil
import com.tenlionsoft.baselib.utils.LogUtils
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.Response
import okhttp3.WebSocket
import okhttp3.WebSocketListener
import okio.ByteString
import java.util.concurrent.TimeUnit
import java.util.concurrent.locks.Lock
import java.util.concurrent.locks.ReentrantLock


class WsManager private constructor(val builder: Builder) : IWsManager {
    private val RECONNECT_INTERVAL: Int = 10 * 1000

    private val RECONNECT_MAX_TIME: Long = (120 * 1000).toLong()

    private
    val PING_TIME: Int = 50
    private var mContext: Context = builder.mContext
    private var wsUrl: String = builder.wsUrl;
    private var mWebSocket: WebSocket? = null
    private var mOkHttpClient: OkHttpClient? = builder.mOkHttpClient
    private var mRequest: Request? = null
    private var mCurrentStatus: Int = WsStatus.DISCONNECTED //websocket连接状态
    private var isNeedReconnect = builder.needReconnect //是否需要断线自动重连
    private var isManualClose = false //是否为手动关闭websocket连接
    private var mLock: Lock = ReentrantLock()
    private val wsMainHandler: Handler = Handler(Looper.getMainLooper())
    private var reconnectCount = 5 //重连次数
    private var mMsgCallBack: MsgCallBack? = builder.callBack
    private val reconnectRunnable = Runnable {
        if (reconnectCount < 10) {
            //重新连接
            val intent = Intent()
//            intent.setAction(PathConfig.ACTION_SOCKET_RELINK)
            mContext.sendBroadcast(intent)
        }
    }


    private val mWebSocketListener: WebSocketListener = object : WebSocketListener() {

        override fun onOpen(webSocket: WebSocket, response: Response) {
            mWebSocket = webSocket
            setCurrentStatus(WsStatus.CONNECTED)
            connected()
            sendBroadcast(ProjectConfig.A_S_CONNECTED)
        }

        override fun onMessage(webSocket: WebSocket, bytes: ByteString) {
            if (Looper.myLooper() != Looper.getMainLooper()) {
                wsMainHandler.post { sendNotify(bytes.toString()) }
            } else {
                sendNotify(bytes.toString())
            }
        }

        override fun onMessage(webSocket: WebSocket, text: String) {
            if (Looper.myLooper() != Looper.getMainLooper()) {
                wsMainHandler.post { sendNotify(text) }
            } else {
                sendNotify(text)
            }
        }

        override fun onClosing(webSocket: WebSocket, code: Int, reason: String) {
            if (Looper.myLooper() != Looper.getMainLooper()) {
                wsMainHandler.post { LogUtils.e("连接关闭中") }
            } else {
                LogUtils.e("连接关闭中")
            }
        }

        override fun onClosed(webSocket: WebSocket, code: Int, reason: String) {
            if (Looper.myLooper() != Looper.getMainLooper()) {
                wsMainHandler.post { LogUtils.e("服务器连接已关闭") }
            } else {
                LogUtils.e("服务器连接已关闭")
            }
        }

        override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
            try {
                tryReconnect()
                //发送失败广播
                sendBroadcast(ProjectConfig.A_S_FAIL)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }


    private fun initWebSocket() {
        if (mOkHttpClient == null) {
            mOkHttpClient =
                OkHttpClient.Builder().pingInterval(PING_TIME.toLong(), TimeUnit.SECONDS)
                    .retryOnConnectionFailure(true).build()
        }
        if (mRequest == null) {
            mRequest = Request.Builder().url(wsUrl).build()
        }
        mOkHttpClient!!.dispatcher.cancelAll()
        try {
            mLock.lockInterruptibly()
            try {
                mOkHttpClient!!.newWebSocket(mRequest!!, mWebSocketListener)
            } finally {
                mLock.unlock()
            }
        } catch (e: InterruptedException) {
            e.printStackTrace()
        }
    }

    //发送广播
    private fun sendBroadcast(action: String) {
        val intent = Intent(action)
        mContext.sendBroadcast(intent)
    }

    override fun getWebSocket(): WebSocket? {
        return mWebSocket
    }


    @Synchronized
    override fun isWsConnected(): Boolean {
        return mCurrentStatus == WsStatus.CONNECTED
    }

    @Synchronized
    override fun getCurrentStatus(): Int {
        return mCurrentStatus
    }

    @Synchronized
    override fun setCurrentStatus(currentStatus: Int) {
        this.mCurrentStatus = currentStatus
    }


    override fun startConnect() {
        isManualClose = false
        buildConnect()
    }

    override fun stopConnect() {
        isManualClose = true
        disconnect()
    }

    private fun tryReconnect() {
        if (!isNeedReconnect or isManualClose) {
            return
        }
        if (!isNetworkConnected(mContext)) {
            setCurrentStatus(WsStatus.DISCONNECTED)
        }
        setCurrentStatus(WsStatus.RECONNECT)
        val delay = (reconnectCount * RECONNECT_INTERVAL).toLong()
        wsMainHandler.postDelayed(reconnectRunnable, delay)
        reconnectCount++
    }

    private fun cancelReconnect() {
        wsMainHandler.removeCallbacks(reconnectRunnable)
        reconnectCount = 0
    }

    private fun connected() {
        cancelReconnect()
    }

    private fun disconnect() {
        if (mCurrentStatus == WsStatus.DISCONNECTED) {
            return
        }
        cancelReconnect()
        mOkHttpClient!!.dispatcher.cancelAll()
        if (mWebSocket != null) {
            val isClosed = mWebSocket!!.close(
                WsStatus.CODE.NORMAL_CLOSE,
                WsStatus.TIP.NORMAL_CLOSE
            )
            //非正常关闭连接
            if (!isClosed) {
//                LogUtils.e("服务器连接失败")
            }
        }
        setCurrentStatus(WsStatus.DISCONNECTED)
    }

    @Synchronized
    private fun buildConnect() {
        if (!isNetworkConnected(mContext)) {
            setCurrentStatus(WsStatus.DISCONNECTED)
        }
        when (getCurrentStatus()) {
            WsStatus.CONNECTED, WsStatus.CONNECTING -> {}
            else -> {
                setCurrentStatus(WsStatus.CONNECTING)
                initWebSocket()
            }
        }
    }

    //发送消息
    override fun sendMessage(msg: String?): Boolean {
        if (TextUtils.isEmpty(msg)) {
            return false;
        } else {
            if (ProjectConfig.IS_SECRET) {
                try {
                    val strMsg: String =
                        AesUtil.aesCommonEncoder(ProjectConfig.SOCKET_MSG_SECRET, msg!!)
                    return send(strMsg)
                } catch (e: Exception) {
                    e.printStackTrace()
                    return false
                }
            } else {
                return send(msg!!)
            }
        }
    }

    override fun sendMessage(byteString: ByteString?): Boolean {
        if (byteString == null) {
            return false;
        } else {
            return send(byteString)
        }
    }

    private fun send(msg: Any): Boolean {
        var isSend = false
        if (mWebSocket != null && mCurrentStatus == WsStatus.CONNECTED) {
            if (msg is String) {
                isSend = mWebSocket!!.send(msg)
            } else if (msg is ByteString) {
                isSend = mWebSocket!!.send(msg)
            }
            if (!isSend) {
                tryReconnect()
            }
        }
        return isSend
    }

    fun sendImg(img: String?) {
    }

    //检查网络是否连接
    public fun isNetworkConnected(context: Context?): Boolean {
        if (context != null) {
            val cm = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
            val activeNetwork: Network = cm.activeNetwork ?: return false
            val capabilities = cm.getNetworkCapabilities(activeNetwork)
            return capabilities != null && capabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
        } else {
            return false;
        }
    }

    private fun sendNotify(jsonStr: String) {
        if (jsonStr.isNotEmpty()) {
            mMsgCallBack?.onCallBackMsg(jsonStr)
        }
    }

    interface MsgCallBack {
        fun onCallBackMsg(str: String)
    }

    class Builder(val mContext: Context) {
        lateinit var wsUrl: String
        var needReconnect: Boolean = true
        var mOkHttpClient: OkHttpClient? = null
        var callBack: MsgCallBack? = null
        fun wsUrl(url: String): Builder {
            wsUrl = url
            return this
        }

        fun client(client: OkHttpClient?): Builder {
            mOkHttpClient = client
            return this
        }

        fun needReconnect(need: Boolean): Builder {
            needReconnect = need
            return this
        }

        fun addCallBack(callBack: MsgCallBack): Builder {
            this.callBack = callBack
            return this
        }

        fun build(): WsManager {
            return WsManager(this)
        }
    }
}